//-------------------------  dataclass -------------------------------------
dataclass::dataclass()
{
	// Setup all the defaults...
	strcpy(file_name,"DEFAULT.WFM");
	strcpy(header,"(No header)");
	strcpy(speed,"High");

	zoom        =1;
	start       =0;// this is an offset used to move the window along the wave
	linemode    =0;
	h_symmetry  =0;
	snap        =0;
	reserved    =1;
	clock_speed =6;
	clockspeed  =2000000;

	hmark[0]   =64;
	hmark[1]  =192;
	vmark[0]  =100;
	vmark[1]  =300;
	vmark[2]  =500;


	clear(0,MAXIMUMDATALENGTH);// Go and reset all data to 127
}

//-------------------------- clear -----------------------------------------
void dataclass::clear(int start, int end)
{
	for(int x=start;x<=end;data[x++]=127); // Set selected data to zero
}

//------------------------- x_2_screenx ------------------------------------
int dataclass::x_2_screenx(int x)
{
	return((x-start)/zoom);// convert data to screen coords
}

//------------------------- screenx_2_x ------------------------------------
int dataclass::screenx_2_x(int x)
{
	return floor((x*zoom)+start);// convert screen coords to data number
}

//------------------------- draw  ------------------------------------------
void dataclass::draw()
{
	int  vscreen[3];// screen versions of markers
	char text[80];  // Temp array to hold printed data

	mouse_control(MOUSE_HIDE,&mouse_x,&mouse_y,&mouse_button);
	lineto(); // If linemode is on, draw line to last point clicked

	for(int x=0;x<3;x++)
	{
		if(vmark[x]<start) vscreen[x]=0;      // = 0 if the marker off screen.
		else vscreen[x]=x_2_screenx(vmark[x]);// Otherwise convert it to screen
	}                                        // coords.

	setlinestyle(SOLID_LINE, 1, 1);
	setfillstyle(1,VALID_WAVE_BACKGROUND); // draw background
	bar(0,0,vscreen[2],255);               // in appropriate colours
														//
	setfillstyle(1,SELECTED_BACKGROUND);   //
	bar(vscreen[0],0,vscreen[1],255);      //
														//
	setfillstyle(1,THE_VOID_BACKGROUND);   //
	bar(vscreen[2],0,625,255);             //

	setcolor(GRATICULE_COLOUR);            //
	line(0,0,639,0);                       //
	line(0,127,639,127);                   //

	setfillstyle(1,BAR_COLOUR);
	bar(0,256,640,268);                    // bar for vert markers
	bar(625,0,640,265);                    // bar for horiz markers



	int barlength=min(625,((float)(screenx_2_x(625)-start)/vmark[2])*625);
	int barstart=min(625-barlength,((float)start/((float)vmark[2]))*625);
	setfillstyle(1,SCALEBAR);
	bar(barstart,258,barstart+barlength,266);              // Where-we-are-in-waveform bar

															// Draw vertical markers
	setcolor(MARKERCOLOUR);            		   //
	line(vscreen[0],0,vscreen[0],258);        //
	line(vscreen[1],0,vscreen[1],258);        //
	line(0,hmark[0],627,hmark[0]);            //
	line(0,hmark[1],627,hmark[1]);            //
															//
	setcolor(MARKERHANDLECOLOUR);             //
	outtextxy(max(0,vscreen[0]-3),258,"^");   //
	outtextxy(max(0,vscreen[1]-3),258,"^");   //
	outtextxy(max(0,vscreen[1]-3),262,"^");   //
	outtextxy(627,hmark[0]-3,"<");            //
	outtextxy(631,hmark[0]-3,"<");            //
	outtextxy(627,hmark[1]-3,"<");            //
															//
	setcolor(LASTMARKERCOLOUR);               //
	line(vscreen[2],0,vscreen[2],258);        //
	outtextxy(max(0,vscreen[2]-11),259,"END");//

	setcolor(TICCOLOUR);                   // Draw in the ruler tics
														//
	for(x=0;x<127;x+=10)                   //
	{                                      //
		line(625,127-x,627,127-x);          //
		line(625,127+x,627,127+x);
	}

	draw_wave();                           // Now go and plot the waveform

	box(3,272,633,97);                     // Draw middle text box,

	setcolor(BLACK);                       // and all the text to go in it.
	outtextxy(10,280,"Marker 1  :");
	outtextxy(10,292,"Marker 2  :");
	outtextxy(10,322,"END Marker:");
	outtextxy(320,280,"Horizontal marker 2:");
	outtextxy(320,292,"Horizontal marker 1:");

	setcolor(RED);
	sprintf(text,"%d",vmark[0]);           // All these values are in red
	outtextxy(98,280,text);
	sprintf(text,"%d",vmark[1]);
	outtextxy(98,292,text);
	sprintf(text,"%d",vmark[2]);
	outtextxy(98,322,text);
	sprintf(text,"%d",127-hmark[0]);
	outtextxy(480,280,text);
	sprintf(text,"%d",127-hmark[1]);
	outtextxy(480,292,text);

	setcolor(BLACK);                        // While everything else is black
	sprintf(text,"Zoom factor: %.2g",zoom);
	outtextxy(320,310,text);

	sprintf(text,"File name: %s",file_name);
	outtextxy(320,346,text);

	sprintf(text,"Header text: %s",header);
	outtextxy(10,358,text);

	if(clock_speed==6)
	{                                      // Work out which speed message
		strcpy(speed,"High (62Hz to 1MHz)");// to show.
		clockspeed=2000000.0;
	}

	if(clock_speed==0)
	{
		strcpy(speed,"Medium (16Hz to 61.999Hz)");
		clockspeed=250000.0;
	}

	if(clock_speed==2)
	{
		strcpy(speed,"Low (0.24Hz to 15.999Hz)");
		clockspeed=7812.5;
	}

	sprintf(text,"Clock speed: %s",speed);
	outtextxy(320,334,text);

	//Dislpay selected length in mili or microseconds
	if(((vmark[1]-vmark[0])*(1.0/clockspeed)*1000000)>1000)
	{
		sprintf(text,"Selected length: %d (%2.3fms)",vmark[1]-vmark[0],(vmark[1]-vmark[0])*(1.0/clockspeed)*1000);
	}
	else
	{
		sprintf(text,"Selected length: %d (%2.1fus)",vmark[1]-vmark[0],(vmark[1]-vmark[0])*(1.0/clockspeed)*1000000);
	}

	outtextxy(10,304,text);

	sprintf(text,"Frequency: %2.2fHz, Period: %2.1fus",clockspeed/vmark[2],vmark[2]*(1.0/clockspeed)*1000000);

	//Dislpay waveform length in mili or microseconds as well
	if((vmark[2]*(1.0/clockspeed)*1000000)>1000)
	{
		sprintf(text,"Frequency: %2.2fHz, Period: %2.3fms",clockspeed/vmark[2],vmark[2]*(1.0/clockspeed)*1000);
	}
	else
	{
		sprintf(text,"Frequency: %2.2fHz, Period: %2.1fus",clockspeed/vmark[2],vmark[2]*(1.0/clockspeed)*1000000);
	}

	outtextxy(10,334,text);



	mouse_control(MOUSE_SHOW,&mouse_x,&mouse_y,&mouse_button);
}

//---------------------------- draw_wave -------------------------------
// Plots the visible bit of waveform on the screen
void dataclass::draw_wave()
{
	int temp,x;

	for(x=0;x<625;x++)// Run through each pixel on the screen
	{
		temp=screenx_2_x(x); // convert it,
		if((temp>MAXIMUMDATALENGTH)||(temp<0)) return;//and exit if off screen
		putpixel(x,data[temp],WAVE_COLOUR); // otherwise plot it
	}
}

//--------------------------- mousecheck ---------------------------------
void dataclass::mousecheck()
{
	if ((mouse_y>267)||(!mouse_button))
	{
		return;    // we aren't wanted, we'll go
	}

	if (mouse_y>255) // so, we've clicked on the vert marker strip..
	{
		move_vert_markers(); // go and do vert marker stuff...
		return;
	}

	if (mouse_x>=625)// // same for horiz markers
	{
		move_horiz_markers();
		return;
	}

	int dataold, datanew;
	unsigned int pointedto;         // data pointed to by mouse_x
	dataold=datanew=screenx_2_x(mouse_x);//We all start off together

	do
	{
		mouse_control(MOUSE_POSITION_BUTTONS,&mouse_x,&mouse_y,&mouse_button);
		pointedto=screenx_2_x(mouse_x); // get the data element clicked on
		if (pointedto>MAXIMUMDATALENGTH)//did we sail off the end of the world?
		{
			draw();                      // If so, refresh and return
			mouse_wait_up();
			return;
		}

		if ((mouse_y<=255)&&(mouse_x<625))// are we in the data area?
		{
			datanew=pointedto; // Good, assign clicked data point to datanew.
			// Hide the mouse
			mouse_control(MOUSE_HIDE,&mouse_x,&mouse_y,&mouse_button);
			if (snap)
			{
				// Snap mode is on, so go and modify mouse_y if we're near a hmarker
				if(abs(hmark[0]-mouse_y)<10) mouse_y=hmark[0];
				else if(abs(hmark[1]-mouse_y)<10) mouse_y=hmark[1];
			}

			// Translate mouse coords into data coords
			if((datanew>=0)&&(datanew<=MAXIMUMDATALENGTH))// make sure it's valid
			{
				if(datanew==dataold)                       // Can't remember what
				{                                          // all this is for, but
					mousedit(datanew,mouse_y);              // beleve me it's all
				}                                          // needed
				else
				{
					draw_line(dataold,data[dataold],datanew,mouse_y);
				}                        // Ah yes, this uses mousedit for single
												 // pixel edits, and line for everthing
				dataold=datanew;         // else. (You work it out -- it made
			}                           // sense to me at the time...)
			mouse_control(MOUSE_SHOW,&mouse_x,&mouse_y,&mouse_button);
		}
	}
	while(mouse_button); // drag'n'draw (tm)

	//for(x=0;x<1000;x++) printf("/nI will comment code as I write it");


	draw();              // refresh screen
}


//-------------------------- mousedit  ---------------------------------
//In which we let the user change the value of any part of the wave on the screen
void dataclass::mousedit(int data_x,int y)
{
	int this_colour;

	// Choose erase colour
	this_colour=VALID_WAVE_BACKGROUND;
	if (data_x>=vmark[0]) this_colour=SELECTED_BACKGROUND;
	if (data_x>=vmark[1]) this_colour=VALID_WAVE_BACKGROUND;
	if (data_x>=vmark[2]) this_colour=THE_VOID_BACKGROUND;

	// Erase this data element
	for(float x=x_2_screenx(data_x);    x<min(625,x_2_screenx(data_x+1));x++)
	{ //      ^screen of this data^  to ^screen of next data^^^^^^^^^^^
		putpixel(x,data[data_x],this_colour); // blank out old data
	}

	// Update the data
	data[data_x]=y; // y is the current mouse_y position passed into this routine

	// Redraw this data element. If we're zoomed in, we'll need to draw several
	// pixels, so...
	for(x=x_2_screenx(data_x);x<min(625,x_2_screenx(data_x+1));x++)
	{
		putpixel(x,data[data_x],WAVE_COLOUR);// ...we draw each of them in.
	}
}

//----------------------------- draw_line -----------------------------------
//Draws a line between x1,y1 - x2,y2
void dataclass::draw_line(int start_x,int start_y,int finish_x,int finish_y)
{  //                          ^x1^        ^y1^        ^x2^         ^y2^
	float rise,run,slope,y;
	int x,temp;

	if(finish_x<start_x)
	{
		temp=finish_x;    // we can only draw lines in one direction, so
		finish_x=start_x; // we swap things around until everything
		start_x=temp;     // points in the right direction.

		temp=finish_y;
		finish_y=start_y;
		start_y=temp;
	}
	y=start_y;

	rise  = finish_y-start_y; // These should be obvious...
	run   = finish_x-start_x;
	slope = (rise/run);

	for(x=start_x; x<=finish_x; x++)
	{
		mousedit(x,y); // Use mousedit to actually do the work
		y+=slope;
	}
}

//--------------------------- zoomin --------------------------------------
// Hmm, this routine is called zoomin, I wonder what it does....
void dataclass::zoomin()
{
	if(zoom>(0.015625)) zoom /=2;// Lets not go overboard now, eh?
}

//--------------------------- zoomout -------------------------------------
//This one is a bit trickier, in that we don't want to zoom out
// into nowheresville, and so we have to wind start back so that the
// righthand side of the screen is in RealSpace.
void dataclass::zoomout()
{
	if(zoom<64) zoom*=2;// zoom==64 gives us the whole wave on the screen
	while( (screenx_2_x(625)<0)||(screenx_2_x(625)>MAXIMUMDATALENGTH) ) start--;
	if (zoom>32) start=0;

}

//--------------------------- scroll_left ---------------------------------
//move the window left a useful amount, depending on the current zoom factor
void dataclass::scroll_left(int distance)
{
	if((start>max(1,distance*zoom))&&(zoom<64)) start-=max(1,distance*zoom);//Can we do it?
	else start=0;//na.
}

//--------------------------- scroll_right ---------------------------------
// Same as above, except that we need to check that we don't wander into
// The Other World...
void dataclass::scroll_right(int distance)
{
	if(zoom<64)// If all the wave fits on the screen, we don't need to scroll
	{
		if(screenx_2_x(625)<MAXIMUMDATALENGTH-(distance*zoom)) start+=max(1,distance*zoom);
		else start=MAXIMUMDATALENGTH-(625*zoom);
	}
}

//--------------------------- move_horiz_markers ---------------------------
void dataclass::move_horiz_markers()
{
	int thismarker=-1;
	int slice[640];

	for(int x=0;x<2;x++)
	{
		if(abs(mouse_y-hmark[x])<5) thismarker=x;// Which one did we click on?
	}

	if(thismarker==-1) return; // We didn't click within 5 pixels of a handle.

	setcolor(YELLOW);//Moving markers are drawn in yellow (I love hard coding)

	do
	{
		mouse_control(MOUSE_POSITION_BUTTONS,&mouse_x,&mouse_y,&mouse_button);
		if(mouse_y>255) mouse_y=255;// keep the marker in the data area

		mouse_control(MOUSE_HIDE,&mouse_x,&mouse_y,&mouse_button);// Mouse, OFF!

		for(int x=0;x<640;x++)
		{
			slice[x]=getpixel(x,mouse_y);//Save the bit of screen under destination
		}
		line(0,mouse_y,640,mouse_y); // draw in temp (yellow) marker
		for(x=0;x<641;x++)
		{
			putpixel(x,mouse_y,slice[x]); // And draw in the old bit of screen,
		}                                // erasing the temp marker as we go.
		mouse_control(MOUSE_SHOW,&mouse_x,&mouse_y,&mouse_button);//Mouse, ON!
	}
	while(mouse_button);// Do all of that while you had the button down

	hmark[thismarker]=mouse_y;//Actually move the marker where you left it

	if(h_symmetry)// Oh yes, forgot this bit...
	{
		if(thismarker==0)
		{
			hmark[1]=127+(127-hmark[0]); //Clumsy, but it works...
		}
		else hmark[0]=127-(hmark[1]-127);
	}


	if(hmark[1]<hmark[0])// Lastly we swap markers if you dragged one past the other
	{
		x=hmark[0];
		hmark[0]=hmark[1];
		hmark[1]=x;
	}

	draw();//Refresh everything
}

//--------------------------- move_vert_markers ---------------------------
//Getting bored typing all these comments, read the last function for details.
void dataclass::move_vert_markers()
{
	int thismarker=-1;
	int slice[257];

	for(int x=2;x>=0;x--)
	{
		if (abs(mouse_x-x_2_screenx(vmark[x]))<10) thismarker=x;
	}

	if (thismarker==-1) return;

	setcolor(YELLOW);

	do
	{
		mouse_control(MOUSE_POSITION_BUTTONS,&mouse_x,&mouse_y,&mouse_button);

			mouse_control(MOUSE_HIDE,&mouse_x,&mouse_y,&mouse_button);
		if (mouse_x>625) mouse_x=625;

			for(int y=0;y<256;y++)
			{
				slice[y]=getpixel(mouse_x,y);
			}
			line(mouse_x,0,mouse_x,255);
			for(y=0;y<256;y++)
			{
				putpixel(mouse_x,y,slice[y]);
			}
			mouse_control(MOUSE_SHOW,&mouse_x,&mouse_y,&mouse_button);
	}
	while(mouse_button);

	vmark[thismarker]=screenx_2_x(mouse_x);

	// did we try to drag it off the end of the world?
	if((vmark[thismarker]>MAXIMUMDATALENGTH)||(vmark[thismarker]<0))vmark[thismarker]=MAXIMUMDATALENGTH;
	if(vmark[1]<vmark[0])
	{
		x=vmark[0];
		vmark[0]=vmark[1];
		vmark[1]=x;
	}

	draw();
}

//------------------------------ cut ---------------------------------------
//Stores selected bit of wave to disk, and then deletes it from the wave
void dataclass::cut()
{
	FILE *output_file;

	output_file = fopen(".\\arbedit.tmp", "wb");

	fprintf(output_file,"%d",vmark[1]-vmark[0]);//write number of bytes to save

	for (int x=vmark[0]; x<vmark[1]; x++)// Walk through selected area
	{
		fprintf(output_file, "%c",data[x]);// Save each byte to disk
	}

	fclose(output_file);

	// Now we move the data from vmarker2 to vmarker2-selected length
	// all the way up to the end, thus stomping all over the cut area.
	for(x=0;x<MAXIMUMDATALENGTH-vmark[1];x++)
	{
		data[vmark[0]+x] = data[vmark[1]+x];
	}

	//The above leaves duplicated data at the very end, so we blank it out.
	clear(MAXIMUMDATALENGTH-vmark[1]-vmark[0],MAXIMUMDATALENGTH);
}

//------------------------------ copy --------------------------------------
//Same as above, except we don't alter anything, just save
void dataclass::copy()
{
	FILE *output_file;

	output_file = fopen(".\\arbedit.tmp", "wb");

	fprintf(output_file,"%d",vmark[1]-vmark[0]);
	for (int x=vmark[0]; x<vmark[1]; x++)
	{
		fprintf(output_file, "%c",data[x]);
	}

	fclose(output_file);
}

//------------------------------ paste --------------------------------------
// Here we insert data from disk into the wave, starting from vmark1
void dataclass::paste()
{
	int number_data;
	FILE *input_file = NULL;

	if(input_file = fopen(".\\arbedit.tmp","rb")) // Open data file.
	{
		fscanf(input_file,"%d", &number_data);//Read in number of bytes to get

		number_data=min(MAXIMUMDATALENGTH-vmark[0],number_data);//Size of block
																				  //to shift
		for(int x=MAXIMUMDATALENGTH-number_data;x>=vmark[0];x--)
		{
			data[x+number_data] = data[x];//Move everyone up a block
		}

		for(x=0; x<number_data;x++)
		{
		  data[vmark[0]+x]=fgetc(input_file);//Past the new stuff into the hole
		}
		vmark[1]=vmark[0]+number_data;
	}else return;
	fclose(input_file); // And close the file.
}

//---------------------------- undo ---------------------------------------
void dataclass::undo()
{
	if (undo_mode)
	{
		FILE *output_file;

		output_file = fopen(".\\arbundo.tmp", "wb");

		for (int x=0;x<MAXIMUMDATALENGTH;x++)
		{
			fprintf(output_file,"%c",data[x]);
		}

		fclose(output_file);
	}
}
//-------------------------- undo_button -----------------------------------
void dataclass::undo_button()
{
	if (undo_mode)
	{
		FILE *input_file = NULL;

		if(input_file = fopen(".\\arbundo.tmp","rb")) // Open data file.
		{
			for(int x=0;x<MAXIMUMDATALENGTH;x++)
			{
				data[x]=fgetc(input_file);
			}
		} else return;
		fclose(input_file); // And close the file.
	}
}


//---------------------------- lineto -------------------------------------
void dataclass::lineto(void)
{
	if(linemode==0)return;
	int first,last;
	int x=vmark[0];

	while(x<=vmark[1])
	{
		while(data[x]==127)
		{
			x++;
		}

		first = x;
		x++;

		while(data[x]==127)
		{
			x++;
		}

		last = x;

		if(last<vmark[1])
		{
			draw_line(first,data[first],last,data[last]);
		}
	}
}

//--------------------------- clear_button ---------------------------------
void dataclass::clear_button(void)
{
	undo();
	clear(vmark[0],vmark[1]);
}

//--------------------------- line_button ---------------------------------
void dataclass::line_button(void)
{
	undo();
	draw_line(vmark[1],hmark[0],vmark[0],hmark[1]);
}

//----------------------------- flip_x ---------------------------------------
void dataclass::flip_x(void)
{

	copy();

	int number_data,x;
	FILE *input_file = NULL;

	if(input_file = fopen(".\\arbedit.tmp","rb")) // Open data file.
	{
		fscanf(input_file,"%d", &number_data);

		for(x=0; x<number_data;x++)
		{
		data[vmark[1]-x-1]=fgetc(input_file);
		}
	} else return;
	fclose(input_file); // And close the file.
}
//----------------------------- flip_x ---------------------------------------

void dataclass::flip_y(void)
{
	for(int x=vmark[0]; x<vmark[1]; x++)
	{
		data[x]=255-data[x];
	}
}

//----------------------------- sine ---------------------------------------
void dataclass::sine(void)
{
	int length=vmark[1]-vmark[0];
	int x,d,baseline,amplitude;

	undo();

	if (vmark[0]<1)
	{
		amplitude=data[0]-hmark[0];;
		baseline=data[0];
	}
	else
	{
		amplitude=data[vmark[0]-1]-hmark[0];
		baseline=data[vmark[0]-1];
	}

	for(x=0;x<length;x++)
	{
		d=baseline-127+(127-amplitude*(sin((x+1)*(6.28/length))));

		if(d>255)d=255;
		if(d<0)d=0;

		data[vmark[0]+x] = (unsigned char)d;
	}
}

//---------------------------- shiftup --------------------------------------
void dataclass::shiftup(void)
{
	for(int x=vmark[0];x<vmark[1];x++)
	{
		if (data[x]>0)	data[x]--;
	}
}

//---------------------------- shiftdown ------------------------------------
void dataclass::shiftdown(void)
{
	for(int x=vmark[0];x<vmark[1];x++)
	{
		if (data[x]<255)	data[x]++;
	}
}

//-----------------------------  load ---------------------------------------
void dataclass::load(char *filename)
{
	FILE *input_file = NULL;
	unsigned char in_buffer[100];
	unsigned char low=0,high=0;
	int i;

	strcpy(file_name,filename);

	if ((input_file = fopen(filename, "rb")) != NULL)
	{
		clear(0,MAXIMUMDATALENGTH);
		fgets(in_buffer,59,input_file);
		strcpy(header,in_buffer);

		low=(unsigned char)fgetc(input_file);
		high=(unsigned char)fgetc(input_file);
		vmark[2] = high * 256 +low;

		low=(unsigned char)fgetc(input_file);
		high=(unsigned char)fgetc(input_file);
		clock_speed = high * 256 +low;

		low=(unsigned char)fgetc(input_file);
		high=(unsigned char)fgetc(input_file);
		reserved    = high * 256 +low;

		for(i=0;i<vmark[2];i++)
		{
			fgets(in_buffer,2,input_file);
			sscanf(in_buffer,"%c",&low);
			data[i]=255-low;
		}

		fclose(input_file);

		vmark[0]= 0;
		vmark[1]= vmark[2]/2;
	}
	else
	{
		fclose(input_file);// Couldn't open file, close it anyway.
	}

	zoom=1;
	start=0;

	while((x_2_screenx(vmark[2])>640)&&(screenx_2_x(640)<MAXIMUMDATALENGTH/2))
	{
		zoom=zoom*2;
	}
}

//-----------------------------  save --------------------------------------
void dataclass::save(void)
{

	FILE *output_file = NULL;
	char *text, short_file_name[50], padded_header[80];
	int i,start,file_name_length;


	strncpy(short_file_name,file_name,strlen(file_name)-4);
	short_file_name[strlen(file_name)-4]=0;// terminate truncated string

	text = get_string("SAVE WAVEFORM","Enter file name:",8,short_file_name);

	if (strcmp(text,"~~~~~")==0) return;// they cancelled! Dive, Dive, Dive!!!

	file_name_length=strlen(text);

	if(file_name_length>0)
	{
		strcpy(file_name,text);
		strcat(file_name,".wfm");

		strcpy(padded_header,header);
		start=strlen(padded_header);

		for(i=start;i<58;i++)
		{
			padded_header[i]= ' ';
		}

		padded_header[58]= 0;

		output_file = fopen(file_name,"wb"); // Open data file
		fputs(padded_header,output_file);

		fprintf(output_file,"%c",(unsigned char)vmark[2]-((vmark[2]/256)*256));
		fprintf(output_file,"%c",(unsigned char)(vmark[2]/256));

		fprintf(output_file,"%c",(unsigned char)clock_speed-((clock_speed/256)*256));
		fprintf(output_file,"%c",(unsigned char)(clock_speed/256));

		fprintf(output_file,"%c",(unsigned char)reserved   -((reserved   /256)*256));
		fprintf(output_file,"%c",(unsigned char)(reserved   /256));

		for(i=0;i<vmark[2];i++)
		{
			fprintf(output_file,"%c",255-data[i]);
		}

		fprintf(output_file,"%c",0);

		fclose(output_file);
	}
}

//----------------------------- zero_cross ----------------------------------
void dataclass::zero_cross(void)
{
	while((data[vmark[1]]==127)&&(vmark[1]<vmark[2]))
		{
			vmark[1]++;
		}

	if(data[vmark[1]]>127)
	{
		while((data[vmark[1]]>127)&&(vmark[1]<vmark[2]))
		{
			vmark[1]++;
		}
	}
	else
	{
		while((data[vmark[1]]<127)&&(vmark[1]<vmark[2]))
		{
			vmark[1]++;
		}
	}

	if(vmark[1]<vmark[0])
	{
		int x=vmark[0];
		vmark[0]=vmark[1];
		vmark[1]=x;
	}
}

//---------------------------- noise ----------------------------------------
void dataclass::noise(void)
{
	int x,rnd;

	undo();

	for(x=vmark[0];x<vmark[1];x++)
	{
		rnd=((random(127-hmark[0]))-((127-hmark[0])/2))*2;// noise, suitably shifted
		data[x]=max(0,min(255,(int)data[x]+rnd));// data + noise, clipped
	}
}

//----------------------------- remove_dc ----------------------------------
void dataclass::remove_dc(void)
{
	float average,total=0;

	undo();

	for(int x=vmark[0];x<vmark[1];x++)
	{
		total+=data[x];
	}

	average=total/(vmark[1]-vmark[0]);

	for(x=vmark[0];x<vmark[1];x++)
	{
		data[x]=min(255,max(0,(int)data[x]+(127-average)));
	}
}

//------------------------------ add ----------------------------------------
void dataclass::add(void)
{
	int number_data,x=0,n=vmark[0],c;
	FILE *input_file = NULL;

	while(n<vmark[1])
	{
	if(input_file = fopen(".\\arbedit.tmp","rb")) // Open data file.
	{
		fscanf(input_file,"%d", &number_data);


		for(x=0; ((x<number_data)&&(x+n<vmark[1]));x++)
		{
			c=fgetc(input_file);
			data[x+n]= min(255,max(0,(int)data[x+n]+c-127));
		}
	} else return;
	n+=x;
	fclose(input_file); // And close the file.
	}
}

//------------------------------ kill_the_rest ----------------------------

void dataclass::kill_the_rest(void)
{
	undo();
	cut();
	clear(0,MAXIMUMDATALENGTH);
	vmark[0]=0;
	paste();
}

//============================ THE END =====================================
