// defines for mouse interface
#define MOUSE_INTERRUPT          0x33 //mouse interrupt number
#define MOUSE_RESET              0x00 // reset the mouse
#define MOUSE_SHOW               0x01 // show the mouse
#define MOUSE_HIDE               0x02 // hide the mouse
#define MOUSE_POSITION_BUTTONS   0x03 // get buttons and postion
#define MOUSE_MOTION_REL         0x0B // query motion counters to compute
#define MOUSE_SET_SENSITIVITY    0x1A // set the sensitivity of mouse
#define MOUSE_LEFT_BUTTON        0x01 // left mouse button mask
#define MOUSE_MIDDLE_BUTTON      0x04 // middle mouse button mask
#define MOUSE_RIGHT_BUTTON       0x02 // right mouse button mask

///////////////////////////////////////////////////////////////////////////////
//---------------------- mouse_control --------------------------------------
int mouse_x, mouse_y, mouse_button;

int mouse_control(int command, int *x, int *y,int *buttons)
{
	union REGS inregs ,outregs;

		switch(command)
		{
			case MOUSE_RESET:
			{
				inregs.x.ax = 0x00;
				int86(MOUSE_INTERRUPT, &inregs, &outregs);
				*buttons = outregs.x.bx;
				return(outregs.x.ax);
			}  break;

			case MOUSE_SHOW:
			{
				inregs.x.ax = 0x01;
				int86(MOUSE_INTERRUPT, &inregs, &outregs);
				return(1);
			}  break;

			case MOUSE_HIDE:
			{
				inregs.x.ax = 0x02;
				int86(MOUSE_INTERRUPT, &inregs, &outregs);
				return(1);
			}  break;

			case MOUSE_POSITION_BUTTONS:
			{
				inregs.x.ax = 0x03;
				int86(MOUSE_INTERRUPT, &inregs, &outregs);
				*x       = outregs.x.cx;
				*y       = outregs.x.dx;
				*buttons = outregs.x.bx;
				return(1);
			}  break;

			case MOUSE_MOTION_REL:
			{
				inregs.x.ax = 0x0B;
				int86(MOUSE_INTERRUPT, &inregs, &outregs);
				*x       = outregs.x.cx;
				*y       = outregs.x.dx;
				return(1);
			}  break;

			case MOUSE_SET_SENSITIVITY:
			{
				inregs.x.ax = 0x1A;
				inregs.x.bx = *x;
				inregs.x.cx = *y;
				inregs.x.dx = *buttons;
				int86(MOUSE_INTERRUPT, &inregs, &outregs);
				return(1);
			}  break;

			default: break;
		}
}
